<?php

namespace Drupal\decoupled_kit\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Url;
use Drupal\Core\Link;
use Drupal\Core\Entity\EntityTypeManagerInterface;

/**
 * Class DefaultForm.
 */
class DefaultForm extends FormBase {

  /**
   * Drupal\Core\Extension\ModuleHandlerInterface definition.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = parent::create($container);
    $instance->moduleHandler = $container->get('module_handler');
    $instance->entityTypeManager = $container->get('entity_type.manager');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'default_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $header = [
      $this->t('Link'),
      $this->t('Url'),
    ];

    if ($this->moduleHandler->moduleExists('decoupled_kit_block')) {
      $url = Url::fromRoute('decoupled_kit_block.index');
      $link = Link::fromTextAndUrl($this->t('Block route'), $url);
      $rows = [];
      $rows[] = ['data' => [$link->toString(), $url->toString() . '?path=/']];
      $form['block_table'] = [
        '#type' => 'table',
        '#caption' => $this->t('Blocks'),
        '#header' => $header,
        '#rows' => $rows,
      ];
    }

    if ($this->moduleHandler->moduleExists('decoupled_kit_menu')) {
      $menus = $this->entityTypeManager->getStorage('menu')->loadMultiple();
      if ($menus) {
        $rows = [];
        foreach ($menus as $key => $value) {
          $url = Url::fromRoute('decoupled_kit_menu.index', ['name' => $key]);
          $link = Link::fromTextAndUrl($value->label(), $url);
          $rows[] = ['data' => [$link->toString(), $url->toString()]];
        }

        $form['menu_table'] = [
          '#type' => 'table',
          '#caption' => $this->t('Menu'),
          '#header' => $header,
          '#rows' => $rows,
        ];
      }
    }

    if ($this->moduleHandler->moduleExists('decoupled_kit_taxonomy')) {
      $vocabs = $this->entityTypeManager->getStorage('taxonomy_vocabulary')->loadMultiple();
      if ($vocabs) {
        $rows = [];
        foreach ($vocabs as $key => $value) {
          $url = Url::fromRoute('decoupled_kit_taxonomy.index', ['name' => $key]);
          $link = Link::fromTextAndUrl($key, $url);
          $rows[] = ['data' => [$link->toString(), $url->toString()]];
        }

        $form['taxonomy_table'] = [
          '#type' => 'table',
          '#caption' => $this->t('Vocabularies'),
          '#header' => $header,
          '#rows' => $rows,
        ];
      }
    }

    if ($this->moduleHandler->moduleExists('decoupled_kit_sitemap')) {
      $url = Url::fromRoute('decoupled_kit_sitemap.index');
      $link = Link::fromTextAndUrl($this->t('Sitemap route'), $url);
      $rows = [];
      $rows[] = ['data' => [$link->toString(), $url->toString()]];
      $form['sitemap_table'] = [
        '#type' => 'table',
        '#caption' => $this->t('Sitemap'),
        '#header' => $header,
        '#rows' => $rows,
      ];
    }

    if ($this->moduleHandler->moduleExists('decoupled_kit_breadcrumb')) {
      $url = Url::fromRoute('decoupled_kit_breadcrumb.index');
      $link = Link::fromTextAndUrl($this->t('Breadcrumb route'), $url);
      $rows = [];
      $rows[] = ['data' => [$link->toString(), $url->toString() . '?path=/']];
      $form['breadcrumb_table'] = [
        '#type' => 'table',
        '#caption' => $this->t('Breadcrumb'),
        '#header' => $header,
        '#rows' => $rows,
      ];
    }

    if ($this->moduleHandler->moduleExists('decoupled_kit_metatag')) {
      $url = Url::fromRoute('decoupled_kit_metatag.index');
      $link = Link::fromTextAndUrl($this->t('Metatag route'), $url);
      $rows = [];
      $rows[] = ['data' => [$link->toString(), $url->toString() . '?path=/']];
      $form['metatag_table'] = [
        '#type' => 'table',
        '#caption' => $this->t('Metatag'),
        '#header' => $header,
        '#rows' => $rows,
      ];
    }

    if ($this->moduleHandler->moduleExists('decoupled_kit_webform')) {
      $url = Url::fromRoute('decoupled_kit_webform.index', ['sid' => 1]);
      $link = Link::fromTextAndUrl($this->t('Webform route'), $url);
      $rows = [];
      $rows[] = ['data' => [$link->toString(), $url->toString()]];
      $form['webform_table'] = [
        '#type' => 'table',
        '#caption' => $this->t('Webform'),
        '#header' => $header,
        '#rows' => $rows,
      ];
    }

    if ($this->moduleHandler->moduleExists('decoupled_kit_redirect')) {
      $url = Url::fromRoute('decoupled_kit_redirect.index');
      $link = Link::fromTextAndUrl($this->t('Redirect route'), $url);
      $rows = [];
      $rows[] = ['data' => [$link->toString(), $url->toString() . '?path=/']];
      $form['redirect_table'] = [
        '#type' => 'table',
        '#caption' => $this->t('Redirect'),
        '#header' => $header,
        '#rows' => $rows,
      ];
    }

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    foreach ($form_state->getValues() as $key => $value) {
      // @TODO: Validate fields.
    }
    parent::validateForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    // Display result.
    foreach ($form_state->getValues() as $key => $value) {
      \Drupal::messenger()->addMessage($key . ': ' . ($key === 'text_format'?$value['value']:$value));
    }
  }

}
